function [mse,Xopt] = ukr_mse(T, model, Xtest)
% ukr_mse : Compute mean squared reconstruction error
% that results from projecting a given dataset onto a 
% ukr manifold. 
%
% function [mse,Xopt] = ukr_mse(T, model [, Xtest])
%
% Inputs :
%   T        - Test data to project onto the manifold 
%              This is either the data matrix itself (columnwise)
%              or a structure of dot product matrices (see ukr_mercer)
%   model    - Trained ukr model structure
%
%   Xtest    - Optional : If you've already calculated the backward
%              mapping, pass the latent variables you've got
%
% Outputs :
%   mse      - Mean squared reconstruction error
%
%   Xopt     - Result of applying the ukr-'backward mapping' as 
%              implicitly defined by the input ukr model to input Ytest
%              (latent variables). If you passed Xtest, Xopt = Xtest
%
% If Kernel UKR is used, this function computes the mean squared
% reconstruction error in feature space.

kernelMode=0;


if ~isstruct(T) 
   if isfield(model,'kernel_o')
      kernelMode=1;
      T=ukr_mercer(model,T);
   end
else
   kernelMode=1;
end

if nargin<3
   Xopt = ukr_g(T,model);
else
   Xopt = Xtest;
end

if kernelMode
   B = ukr_b(Xopt,model);
   mse = sum(B.*(model.G*B)) - 2*sum(B.*T.D) + T.d;
   mse = sum(mse,2)/size(Xopt,2);
else 
   Yrec = ukr_f(Xopt,model);
   mse = sum(sum((Yrec-T).^2))/size(Xopt,2);
end
