function index = ukr_nfr(T, model)
% UKR_NFR Determine 'nearest feature space reconstruction' for observable space element.
%
% index = ukr_nfr(Y, model)
% index = ukr_nfr(T, model)
%
% Inputs: 
%   Y      -  Observable data space elements.
%             This is either a structure with elements D and d 
%             or a matrix of observable data in columns ( not in feature space)
%
%   T      -  Structure as returned by ukr_mercer
%   T.D    -  Matrix of feature space dot products between training and
%             test data
%   T.d    -  (optional here) Row vector of feature space norms of the test data
%
%   model  -  Trained ukr_model structure.
%
% Outputs:
%   index  -  The corresponding latent space element's index with regard to the 
%             model's latent data matrix.

error(nargchk(2, 2, nargin)) 

if isstruct(T) 
    [N M]=size(T.D);
    if size(model.G,1)~=N
        error('Dimensions of dot product matrices are not correct');
    end
else 
    [d1 M]=size(T);
    [d2 N]=size(model.Y);
    
    if (d1~=d2) error('Dimensions (training/test data) do not match'); end;
    
    if isfield(model,'kernel_o')
        T=ukr_mercer(model,T); 
    else
        error('No model.kernel_o given.');
    end
end

if ~isfield(model,'G')
    error('Matrix G of dot products in feature space is not part of model structure.');
end

B = ukr_b(model.X,model);

% calculate feature space norm of reconstructed data
r = full(sum(B.*(model.G*B)));
D = repmat(r',1,M) - 2*B'*T.D;
[md index]=min(D);

